@@ -32,6 +32,44 @@ class VectorizerLayer(Layer):
             return res
         return np.array(res.todense())
 
+    def get_graph(
+        self,
+        documents,
+        threshold=.1,
+        pmi_threshold=.5,
+        window_size=5,
+        k=1
+    ):
+
+        import networkx as nx
+        from convectors.linguistics import pmi
+
+        edges = []
+        X = self.vectorizer.transform(documents)
+        features = self.vectorizer.get_feature_names_out()
+        features_set = set(features)
+        for i in range(len(documents)):
+            start, end = X.indptr[i], X.indptr[i+1]
+            for ind, w in zip(X.indices[start:end], X.data[start:end]):
+                if w < threshold:
+                    continue
+                word = features[ind]
+                edges.append((i, word, w))
+
+        pmi_ = pmi(documents,
+                   undirected=True,
+                   threshold=pmi_threshold,
+                   window_size=window_size,
+                   k=k)
+        for (a, b), w in pmi_.items():
+            if a in features_set and b in features_set:
+                edges.append((a, b, w))
+
+        G = nx.Graph()
+        G.add_nodes_from(range(len(documents)))
+        G.add_weighted_edges_from(edges)
+        return G
+
 
 class TfIdf(VectorizerLayer):
     def __init__(
@@ -70,13 +108,18 @@ class OddsVectorizer(Layer):
         input=None,
         output=None,
         max_features=None,
-        sparse=True,
+        norm="l2",
+        threshold=2,
+        ci=.95,
         name=None,
-        verbose=True,
-        **kwargs,
+        verbose=True
     ):
+        from scipy.stats import norm as scipy_norm
         super().__init__(input, output, name, verbose)
         self.max_features = max_features
+        self.norm = norm
+        self.threshold = threshold
+        self.ci_factor = -scipy_norm.isf((1 - ci) / 2.)
 
     def fit(self, documents, y=None):
         import itertools
@@ -107,11 +150,17 @@ class OddsVectorizer(Layer):
         X = csr_matrix((data, (xs, ys)),
                        dtype=float,
                        shape=(len(series), self.dim))
-        normalize(X, axis=1, norm="l2", copy=False)
+        if self.norm is not None:
+            normalize(X, axis=1, norm=self.norm, copy=False)
         return X
 
     def get_graph(
-        self, documents, threshold=2, pmi_threshold=.5, window_size=5
+        self,
+        documents,
+        threshold=2,
+        pmi_threshold=.5,
+        window_size=5,
+        k=1
     ):
         from collections import Counter
 
@@ -121,54 +170,59 @@ class OddsVectorizer(Layer):
         edges = []
         for i, doc in enumerate(documents):
             count = Counter(doc)
-            count_total = sum(count.values())
+            doc_count = sum(count.values())
             for word, tf in count.items():
                 if word not in self.tf:
                     continue
-                odds = self.get_low_odds(
-                    tf, count_total, self.tf[word], self.total_count)
+                odds = self.compute_odds(
+                    tf, doc_count, self.tf[word], self.total_count)
                 if odds < threshold:
                     continue
                 edges.append((i, word, np.log(odds)))
 
-        pmi_ = pmi(documents, undirected=True,
-                   threshold=pmi_threshold, window_size=window_size)
+        pmi_ = pmi(documents,
+                   undirected=True,
+                   threshold=pmi_threshold,
+                   window_size=window_size,
+                   k=k)
         for (a, b), w in pmi_.items():
             if a in self.tf and b in self.tf:
                 edges.append((a, b, w))
 
         G = nx.Graph()
+        G.add_nodes_from(range(len(documents)))
         G.add_weighted_edges_from(edges)
         return G
 
     def vectorize(self, doc, i):
         from collections import Counter
         count = Counter(doc)
-        count_total = sum(count.values())
+        doc_count = sum(count.values())
         xs, ys, data = [], [], []
         for word, tf in count.items():
             if word not in self.tf:
                 continue
-            odds = self.get_low_odds(
-                tf, count_total, self.tf[word], self.total_count)
-            if odds <= 1:
+            odds = self.compute_odds(
+                tf, doc_count, self.tf[word], self.total_count)
+            if odds <= self.threshold:
                 continue
             xs.append(i)
             ys.append(self.token2id[word])
             data.append(np.log(odds))
+            # data.append(odds)
+
         return xs, ys, data
 
-    def get_low_odds(self, a, c, b, d):
-        eps = 1e-6
-        c = max(c - a, eps)
-        d = max(d - b, eps)
-        a = max(a, eps)
-        b = max(b, eps)
-        odds_ratio = (a/c) / (b/d)
+    def compute_odds(self, a, b, c, d):
+        eps = 1
+        b = max(b - a, eps)
+        d = max(d - c, eps)
+        odds_ratio = (a / b) / (c / d)
 
-        uncertainty = np.sqrt(1/a+1/b+1/c+1/d)
-        low = odds_ratio * np.exp(-1.96*uncertainty)
-        return low
+        uncertainty = np.sqrt(1/a + 1/b + 1/c + 1/d)
+        uncertainty = np.exp(self.ci_factor*uncertainty)
+        odds_ratio *= uncertainty
+        return odds_ratio
 
 
 class CountVectorizer(VectorizerLayer):
