@@ -1,6 +1,7 @@
 from .classifier import (MLP, RNN, SVM, TFMLP, AdaBoost, Keras, RandomForest,
                          Transformer, XGBoost)
-from .embedding import CountVectorizer, Doc2Vec, OneHot, Sequence, TfIdf
+from .embedding import (CountVectorizer, Doc2Vec, OddsVectorizer, OneHot,
+                        Sequence, TfIdf)
 from .linguistics import (NER, Contract, FindAll, LangDetect, Lemmatize, NGram,
                           Phrase, Snowball, Sub, Tokenize)
 from .multi import Merge
@@ -16,5 +17,5 @@ __all__ = [
     "RandomForest", "AdaBoost", "SVM", "MLP", "Keras", "Transformer",
     "XGBoost", "Merge", "Normalize", "SplitHashtag", "TFMLP",
     "MLPRegressor", "KerasRegressor", "Lambda", "RNN", "DomainName",
-    "RandomizedSVD", "NER", "Argmax"
+    "RandomizedSVD", "NER", "Argmax", "OddsVectorizer"
 ]
