@@ -523,6 +523,61 @@ class LangDetect(Layer):
             return None
 
 
+class FreqFilter(Layer):
+    parallel = True
+    trainable = True
+    document_wise = True
+
+    def __init__(
+        self,
+        input=None,
+        output=None,
+        name=None,
+        min_df=None,
+        max_df=None,
+        verbose=True,
+        parallel=False
+    ):
+        super().__init__(input, output, name, verbose, parallel)
+        self.max_df = max_df
+        self.min_df = min_df
+
+    def fit(self, series, y=None):
+        from collections import Counter
+        tf = Counter()
+        df = Counter()
+        n_tokens = 0
+        n_docs = len(series)
+
+        for doc in series:
+            visited = set()
+            n_tokens += len(doc)
+            for token in doc:
+                if token not in visited:
+                    df[token] += 1
+                tf[token] += 1
+
+        self.tf = tf
+        self.df = df
+        if isinstance(self.max_df, float):
+            self.max_df = int(round(self.max_df * n_docs))
+        if isinstance(self.min_df, float):
+            self.min_df = int(round(self.min_df * n_docs))
+
+    def process_doc(self, doc):
+        res = []
+        for token in doc:
+            token_df = self.df[token]
+            if self.max_df is not None:
+                if token_df > self.max_df:
+                    continue
+            if self.min_df is not None:
+                if token_df < self.min_df:
+                    continue
+            res.append(token)
+        return res
+
+
 # =============================================================================
 # Functions
 # =============================================================================
@@ -586,18 +641,19 @@ def tokenize(
     # remove stopwords
     if stopwords is not None:
         if not sentence_tokenize:
-            words = [w for w in words if w not in stopwords]
+            words = [w.strip() for w in words if w.strip() not in stopwords]
         else:
-            words = [[w for w in sentence if w not in stopwords]
+            words = [[w.strip() for w in sentence
+                      if w.strip() not in stopwords]
                      for sentence in words]
     return words
 
 
 def pmi(
-    series, window_size=3, undirected=False, min_cooc=2,
+    series, window_size=3, undirected=False, min_cooc=2, k=1,
     threshold=None, normalize=True
 ):
-    from ..utils import PairCounter
+    from ..collections import PairCounter
     pair_counts = PairCounter(undirected=undirected)
     pair_counts.count(series)
     for doc in series:
@@ -607,7 +663,10 @@ def pmi(
                 tgt = window[i]
                 pair_counts.increment(src, tgt)
     edges = pair_counts.get_pmi(
-        normalize=normalize, threshold=threshold, min_cooc=min_cooc)
+        normalize=normalize,
+        threshold=threshold,
+        min_cooc=min_cooc,
+        k=k)
     return edges
 
 
